# Copyright (C) 2020-Present the hyssop authors and contributors.
#
# This module is part of hyssop and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

'''
File created: November 9th 2020

Modified By: hsky77
Last Updated: November 21st 2020 18:05:42 pm
'''

from .command import CommandProcessor

CommandProcessor().process_command()
