# Copyright (C) 2020-Present the hyssop authors and contributors.
#
# This module is part of hyssop and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

'''
File created: August 21st 2020

Modified By: hsky77
Last Updated: November 21st 2020 20:52:41 pm
'''

# reserved folder names
Component_Module_Folder = 'component'
Controller_Module_Folder = 'controller'
Unittest_Module_Folder = 'unit_test'
Dependency_Folder = 'dependency'

# reserved file names
Localization_File = 'local.csv'
Project_Config_File = 'project_config.yml'
Project_Pack_File = 'pack.yml'
Project_Requirement_File = 'requirements.txt'

# project
LocalCode_Not_Subclass = 100
LocalCode_File_Not_Found = 101

# project config
LocalCode_Parameter_Required = 110
LocalCode_Parameter_Type_Error = 111
LocalCode_Setup_Error = 112
LocalCode_Invalid_Parameter = 113
LocalCode_Copied_Name_Equal_To_Origin = 114

# components
LocalCode_Component_Duplicated_Key = 120
LocalCode_Failed_To_Load_Component = 121
LocalCode_Component_Type_Not_Exist = 122

# controllers' localization code
LocalCode_Failed_To_Load_Controller = 130

# controller
LocalCode_Missing_Required_Parameter = 140
LocalCode_Upload_Success = 141

LocalCode_Connect_Failed = 150
